/*******************************************************************************
* File Name: ISR_PWMCompare_intc.c  
* Version 1.20
*
*  Description:
*   API for controlling the state of an interrupt.
*
*
*  Note:
*
********************************************************************************
* Copyright 2008-2009, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/
#include <CYDEVICE.H>
#include <ISR_PWMCompare.H>


/*******************************************************************************
*  Place your includes, defines and code here 
********************************************************************************/
/* `#START ISR_PWMCompare_intc` */
#include "PWM_Window.h"

uint8 PWMCompare_occured;
/* `#END` */


#define CYINT_IRQ_BASE	16

/* Declared in startup, used to set unused interrupts to. */
void IntDefaultHandler(void);

/*******************************************************************************
* Function Name: ISR_PWMCompare_Start
********************************************************************************
* Summary:
*  Set up the interrupt and enable it.
*
* Parameters:  
*   void.
*
*
* Return:
*   void.
*
*******************************************************************************/
void ISR_PWMCompare_Start(void)
{
    /* For all we know the interrupt is active. */
    ISR_PWMCompare_Disable();

    /* Set the ISR to point to the ISR_PWMCompare Interrupt. */
    ISR_PWMCompare_SetVector(ISR_PWMCompare_Interrupt);

    /* Set the priority. */
    ISR_PWMCompare_SetPriority(ISR_PWMCompare_INTC_PRIOR_NUMBER);

    /* Enable it. */
    ISR_PWMCompare_Enable();
}

/*******************************************************************************
* Function Name: ISR_PWMCompare_Stop
********************************************************************************
* Summary:
*   Disables and removes the interrupt.
*
* Parameters:  
*
*
* Return:
*   void.
*
*******************************************************************************/
void ISR_PWMCompare_Stop(void)
{
    /* Disable this interrupt. */
    ISR_PWMCompare_Disable();

    /* Set the ISR to point to the passive one. */
    ISR_PWMCompare_SetVector(IntDefaultHandler);
}

/*******************************************************************************
* Function Name: ISR_PWMCompare_Interrupt
********************************************************************************
* Summary:
*   The default Interrupt Service Routine for ISR_PWMCompare.
*
*   Add custom code between the coments to keep the next version of this file
*   from over writting your code.
*
*
*
* Parameters:  
*
*
* Return:
*   void.
*
*******************************************************************************/
void ISR_PWMCompare_Interrupt(void)
{
    /*  Place your Interrupt code here. */
    /* `#START ISR_PWMCompare_Interrupt` */
	/* Call the ReadStatusRegister to clear the Interrupt Status bit */
	PWM_Window_ReadStatusRegister();
	
	/* Set the flag to indicate occurence of the interrupt */
	PWMCompare_occured = 1;
    /* `#END` */
}

/*******************************************************************************
* Function Name: ISR_PWMCompare_SetVector
********************************************************************************
* Summary:
*   Change the ISR vector for the Interrupt.
*
*
* Parameters:
*   address: Address of the ISR to set in the interrupt vector table.
*
*
* Return:
*   void.
*
*
*******************************************************************************/
void ISR_PWMCompare_SetVector(cyisraddress address)
{
    cyisraddress * ramVectorTable;

    ramVectorTable = (cyisraddress *) *CYINT_VECTORS;

    ramVectorTable[CYINT_IRQ_BASE + ISR_PWMCompare__INTC_NUMBER] = address;
}

/*******************************************************************************
* Function Name: ISR_PWMCompare_GetVector
********************************************************************************
* Summary:
*   Gets the "address" of the current ISR vector for the Interrupt.
*
*
* Parameters:
*   void.
*
*
* Return:
*   Address of the ISR in the interrupt vector table.
*
*
*******************************************************************************/
cyisraddress ISR_PWMCompare_GetVector(void)
{
    cyisraddress * ramVectorTable;

    ramVectorTable = (cyisraddress *) *CYINT_VECTORS;

    return (cyisraddress) ramVectorTable[CYINT_IRQ_BASE + ISR_PWMCompare__INTC_NUMBER];
}

/*******************************************************************************
* Function Name: ISR_PWMCompare_SetPriority
********************************************************************************
* Summary:
*   Sets the Priority of the Interrupt.
*
*
* Parameters:
*   priority: Priority of the interrupt. 0 - 7, 0 being the highest.
*
*
* Return:
*   void.
*
*
*******************************************************************************/
void ISR_PWMCompare_SetPriority(uint8 priority)
{
    *ISR_PWMCompare_INTC_PRIOR = priority << 5;
}

/*******************************************************************************
* Function Name: ISR_PWMCompare_GetPriority
********************************************************************************
* Summary:
*   Gets the Priority of the Interrupt.
*
*
* Parameters:
*   void.
*
*
* Return:
*   Priority of the interrupt. 0 - 7, 0 being the highest.
*
*
*******************************************************************************/
uint8 ISR_PWMCompare_GetPriority(void)
{
    uint8 priority;


    priority = *ISR_PWMCompare_INTC_PRIOR >> 5;

    return priority;   
}

/*******************************************************************************
* Function Name: ISR_PWMCompare_Enable
********************************************************************************
* Summary:
*   Enables the interrupt.
*
*
* Parameters:
*   void.
*
*
* Return:
*   void.
*
*
*******************************************************************************/
void ISR_PWMCompare_Enable(void)
{
    /* Enable the general interrupt. */
    *ISR_PWMCompare_INTC_SET_EN = ISR_PWMCompare__INTC_MASK;
}

/*******************************************************************************
* Function Name: ISR_PWMCompare_GetState
********************************************************************************
* Summary:
*   Gets the state (enabled, disabled) of the Interrupt.
*
*
* Parameters:
*   void.
*
*
* Return:
*   1 if enabled, 0 if disabled.
*
*
*******************************************************************************/
uint8 ISR_PWMCompare_GetState(void)
{
    /* Get the state of the general interrupt. */
    return (*ISR_PWMCompare_INTC_SET_EN & ISR_PWMCompare__INTC_MASK) ? 1:0;
}

/*******************************************************************************
* Function Name: ISR_PWMCompare_Disable
********************************************************************************
* Summary:
*   Disables the Interrupt.
*
*
* Parameters:
*   void.
*
*
* Return:
*   void.
*
*
*******************************************************************************/
void ISR_PWMCompare_Disable(void)
{
    /* Disable the general interrupt. */
    *ISR_PWMCompare_INTC_CLR_EN = ISR_PWMCompare__INTC_MASK;
}

/*******************************************************************************
* Function Name: ISR_PWMCompare_SetPending
********************************************************************************
* Summary:
*   Causes the Interrupt to enter the pending state, a software method of
*   generating the interrupt.
*
*
* Parameters:
*   void.
*
*
* Return:
*   void.
*
*
*******************************************************************************/
void ISR_PWMCompare_SetPending(void)
{
    *ISR_PWMCompare_INTC_SET_PD = ISR_PWMCompare__INTC_MASK;
}

/*******************************************************************************
* Function Name: ISR_PWMCompare_ClearPending
********************************************************************************
* Summary:
*   Clears a pending interrupt.
*
*
* Parameters:
*   void.
*
*
* Return:
*   void.
*
*
*******************************************************************************/
void ISR_PWMCompare_ClearPending(void)
{
    *ISR_PWMCompare_INTC_CLR_PD = ISR_PWMCompare__INTC_MASK;
}

