/*******************************************************************************
* File Name: Pin_InputSignal.c  
* Version 1.50
*
* Description:
*  This file contains API to enable firmware control of a Pins component.
*
* Note:
*
********************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/

#include "cytypes.h"
#include "Pin_InputSignal.h"


/*******************************************************************************
* Function Name: Pin_InputSignal_Write
********************************************************************************
* Summary:
*  Assign a new value to the digital port's data output register.  
*
* Parameters:  
*  prtValue:  The value to be assigned to the Digital Port. 
*
* Return: 
*  void 
*  
*******************************************************************************/
void Pin_InputSignal_Write(uint8 value) 
{
    uint8 staticBits = Pin_InputSignal_DR & ~Pin_InputSignal_MASK;
    Pin_InputSignal_DR = staticBits | ((value << Pin_InputSignal_SHIFT) & Pin_InputSignal_MASK);
}


/*******************************************************************************
* Function Name: Pin_InputSignal_SetDriveMode
********************************************************************************
* Summary:
*  Change the drive mode on the pins of the port.
* 
* Parameters:  
*  mode:  Change the pins to this drive mode.
*
* Return: 
*  void
*
*******************************************************************************/
void Pin_InputSignal_SetDriveMode(uint8 mode) 
{
	CyPins_SetPinDriveMode(Pin_InputSignal_0, mode);
}


/*******************************************************************************
* Function Name: Pin_InputSignal_Read
********************************************************************************
* Summary:
*  Read the current value on the pins of the Digital Port in right justified 
*  form.
*
* Parameters:  
*  void 
*
* Return: 
*  Returns the current value of the Digital Port as a right justified number
*  
* Note:
*  Macro Pin_InputSignal_ReadPS calls this function. 
*  
*******************************************************************************/
uint8 Pin_InputSignal_Read(void) 
{
    return (Pin_InputSignal_PS & Pin_InputSignal_MASK) >> Pin_InputSignal_SHIFT;
}


/*******************************************************************************
* Function Name: Pin_InputSignal_ReadDataReg
********************************************************************************
* Summary:
*  Read the current value assigned to a Digital Port's data output register
*
* Parameters:  
*  void 
*
* Return: 
*  Returns the current value assigned to the Digital Port's data output register
*  
*******************************************************************************/
uint8 Pin_InputSignal_ReadDataReg(void) 
{
    return (Pin_InputSignal_DR & Pin_InputSignal_MASK) >> Pin_InputSignal_SHIFT;
}


/* If Interrupts Are Enabled for this Pins component */ 
#if defined(Pin_InputSignal_INTSTAT) 

    /*******************************************************************************
    * Function Name: Pin_InputSignal_ClearInterrupt
    ********************************************************************************
    * Summary:
    *  Clears any active interrupts attached to port and returns the value of the 
    *  interrupt status register.
    *
    * Parameters:  
    *  void 
    *
    * Return: 
    *  Returns the value of the interrupt status register
    *  
    *******************************************************************************/
    uint8 Pin_InputSignal_ClearInterrupt(void) 
    {
        return (Pin_InputSignal_INTSTAT & Pin_InputSignal_MASK) >> Pin_InputSignal_SHIFT;
    }

#endif /* If Interrupts Are Enabled for this Pins component */ 


/* [] END OF FILE */ 
